﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Windows.Forms;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using System.Text;
using HeuristicLab.Data;

namespace HeuristicLab.Hive.ExperimentManager.Views {
  [View("HiveJob View")]
  [Content(typeof(HiveJob), true)]
  public partial class HiveJobView : ItemView {
    public new HiveJob Content {
      get { return (HiveJob)base.Content; }
      set {
        if (base.Content != value) {
          base.Content = value;
        }
      }
    }

    public HiveJobView() {
      InitializeComponent();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.JobChanged += new EventHandler(Content_JobChanged);
      Content.JobDtoChanged += new EventHandler(Content_JobDtoChanged);
      Content.JobStateChanged += new EventHandler(Content_JobStateChanged);
    }

    protected override void DeregisterContentEvents() {
      Content.JobChanged -= new EventHandler(Content_JobChanged);
      Content.JobDtoChanged -= new EventHandler(Content_JobDtoChanged);
      Content.JobStateChanged -= new EventHandler(Content_JobStateChanged);
      base.DeregisterContentEvents();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content != null && Content.Job != null) {
        logView.Content = Content.Job.Log;
        childHiveJobView.Content = Content.ChildHiveJobs;
        computeInParallelCheckBox.Checked = Content.Job.ComputeInParallel;
      } else {
        logView.Content = null;
        childHiveJobView.Content = null;
        computeInParallelCheckBox.Checked = false;
      }
      Content_JobChanged(this, EventArgs.Empty);
      Content_JobDtoChanged(this, EventArgs.Empty);
      Content_JobStateChanged(this, EventArgs.Empty);
    }
    
    private void RegisterJobEvents() {
      if (Content != null && Content.Job != null) {
        Content.Job.ComputeInParallelChanged += new EventHandler(Job_ComputeInParallelChanged);
        Content.Job.OptimizerChanged += new EventHandler(Job_OptimizerChanged);
      }
    }

    private void DeregisterJobEvents() {
      if (Content != null && Content.Job != null) {
        Content.Job.ComputeInParallelChanged -= new EventHandler(Job_ComputeInParallelChanged);
        Content.Job.OptimizerChanged -= new EventHandler(Job_OptimizerChanged);
      }
    }

    #region Content Events  
    private void Content_JobChanged(object sender, EventArgs e) {
      RegisterJobEvents();
      Job_OptimizerChanged(this, e);
    }

    private void Content_JobDtoChanged(object sender, EventArgs e) {
      if (InvokeRequired) {
        Invoke(new EventHandler(Content_JobDtoChanged), sender, e);
      } else {
        if (Content != null && Content.JobDto != null) {
          this.jobIdTextBox.Text = Content.JobDto.Id.ToString();
          this.dateCreatedTextBox.Text = Content.JobDto.DateCreated.ToString();
          this.priorityTextBox.Text = Content.JobDto.Priority.ToString();
          this.coresNeededTextBox.Text = Content.JobDto.CoresNeeded.ToString();
          this.memoryNeededTextBox.Text = Content.JobDto.MemoryNeeded.ToString();
        } else {
          this.jobIdTextBox.Text = string.Empty;
          this.dateCreatedTextBox.Text = string.Empty;
          this.priorityTextBox.Text = string.Empty;
          this.coresNeededTextBox.Text = string.Empty;
          this.memoryNeededTextBox.Text = string.Empty;
        }
      }
    }

    private void Content_JobStateChanged(object sender, EventArgs e) {
      if (InvokeRequired) {
        Invoke(new EventHandler(Content_JobStateChanged), sender, e);
      } else {
        if (Content != null && Content.JobDto != null) {
          this.stateTextBox.Text = Content.JobDto.State.ToString();
          this.executionTimeTextBox.Text = Content.JobDto.ExecutionTime.ToString();
          this.dateCalculatedText.Text = Content.JobDto.DateCalculated.ToString();
          this.dateFinishedTextBox.Text = Content.JobDto.DateFinished.ToString();
          this.exceptionTextBox.Text = Content.JobDto.Exception;
        } else {
          this.stateTextBox.Text = string.Empty;
          this.executionTimeTextBox.Text = string.Empty;
          this.dateCalculatedText.Text = string.Empty;
          this.dateFinishedTextBox.Text = string.Empty;
          this.exceptionTextBox.Text = string.Empty;
        }
      }
    }
    
    private void Job_OptimizerChanged(object sender, EventArgs e) {
      if (Content != null && Content.Job != null && Content.Job.Optimizer != null) {
        optimizerNamedItemView.Content = Content.Job.Optimizer;
        runCollectionViewHost.Content = Content.Job.Optimizer.Runs;
      } else {
        optimizerNamedItemView.Content = null;
        runCollectionViewHost.Content = null;
      }
    }

    private void Job_ComputeInParallelChanged(object sender, EventArgs e) {
      if (InvokeRequired) {
        Invoke(new EventHandler(Job_ComputeInParallelChanged), sender, e);
      } else {
        computeInParallelCheckBox.Checked = Content.Job.ComputeInParallel;
        ShowOrHideChildJobTabPage();
      }
    }

    private void ShowOrHideChildJobTabPage() {
      if (Content != null && Content.Job != null) {
        if (!Content.Job.ComputeInParallel) {
          if (tabControl.TabPages.Contains(childJobsTabPage))
            tabControl.TabPages.Remove(childJobsTabPage);
        } else {
          if (!tabControl.TabPages.Contains(childJobsTabPage))
            tabControl.TabPages.Insert(1, childJobsTabPage);
        }
      }
    }
    #endregion

    protected override void SetEnabledStateOfControls() {
      if (InvokeRequired) {
        this.InvokeIfRequired(c => { SetEnabledStateOfControls(); });
      } else {
        base.SetEnabledStateOfControls();
        this.jobIdTextBox.ReadOnly = true;
        this.stateTextBox.ReadOnly = true;
        this.executionTimeTextBox.ReadOnly = true;
        this.dateCreatedTextBox.ReadOnly = true;
        this.dateCalculatedText.ReadOnly = true;
        this.dateFinishedTextBox.ReadOnly = true;
        this.exceptionTextBox.ReadOnly = true;

        this.priorityTextBox.ReadOnly = this.ReadOnly;
        this.coresNeededTextBox.ReadOnly = this.ReadOnly;
        this.memoryNeededTextBox.ReadOnly = this.ReadOnly;
        this.computeInParallelCheckBox.Enabled = !this.ReadOnly && this.Content != null && this.Content.Job != null && this.Content.Job.IsParallelizable;

        optimizerNamedItemView.ReadOnly = true;
        ShowOrHideChildJobTabPage();
      }
    }

    #region Control Events
    private void computeInParallelCheckBox_CheckedChanged(object sender, EventArgs e) {
      if (Content != null) this.Content.Job.ComputeInParallel = this.computeInParallelCheckBox.Checked;
    }

    private void coresNeededTextBox_Validating(object sender, System.ComponentModel.CancelEventArgs e) {
      int val;
      if ((!string.IsNullOrEmpty(coresNeededTextBox.Text)) && (!int.TryParse(coresNeededTextBox.Text, out val))) {
        e.Cancel = true;
        StringBuilder sb = new StringBuilder();
        sb.Append("Invalid Value (Valid Value Format: \"");
        sb.Append(FormatPatterns.GetIntFormatPattern());
        sb.Append("\")");
        errorProvider.SetError(coresNeededTextBox, sb.ToString());
        coresNeededTextBox.SelectAll();
      }
    }

    private void coresNeededTextBox_Validated(object sender, EventArgs e) {
      if (string.IsNullOrEmpty(coresNeededTextBox.Text)) Content.JobDto.CoresNeeded = 1;
      else Content.JobDto.CoresNeeded = int.Parse(coresNeededTextBox.Text);
      errorProvider.SetError(coresNeededTextBox, string.Empty);
    }

    private void memoryNeededTextBox_Validating(object sender, System.ComponentModel.CancelEventArgs e) {
      int val;
      if ((!string.IsNullOrEmpty(memoryNeededTextBox.Text)) && (!int.TryParse(memoryNeededTextBox.Text, out val))) {
        e.Cancel = true;
        StringBuilder sb = new StringBuilder();
        sb.Append("Invalid Value (Valid Value Format: \"");
        sb.Append(FormatPatterns.GetIntFormatPattern());
        sb.Append("\")");
        errorProvider.SetError(memoryNeededTextBox, sb.ToString());
        memoryNeededTextBox.SelectAll();
      }
    }

    private void memoryNeededTextBox_Validated(object sender, EventArgs e) {
      if (string.IsNullOrEmpty(memoryNeededTextBox.Text)) Content.JobDto.MemoryNeeded = 0;
      else Content.JobDto.MemoryNeeded = int.Parse(memoryNeededTextBox.Text);
      errorProvider.SetError(memoryNeededTextBox, string.Empty);
    }

    private void priorityTextBox_Validating(object sender, System.ComponentModel.CancelEventArgs e) {
      int val;
      if ((!string.IsNullOrEmpty(priorityTextBox.Text)) && (!int.TryParse(priorityTextBox.Text, out val))) {
        e.Cancel = true;
        StringBuilder sb = new StringBuilder();
        sb.Append("Invalid Value (Valid Value Format: \"");
        sb.Append(FormatPatterns.GetIntFormatPattern());
        sb.Append("\")");
        errorProvider.SetError(priorityTextBox, sb.ToString());
        priorityTextBox.SelectAll();
      }
    }

    private void priorityTextBox_Validated(object sender, EventArgs e) {
      if (string.IsNullOrEmpty(priorityTextBox.Text)) Content.JobDto.Priority = 0;
      else Content.JobDto.Priority = int.Parse(priorityTextBox.Text);
      errorProvider.SetError(priorityTextBox, string.Empty);
    }
    #endregion

  }
}
